#include "utils.h"

NUtils::FileType NUtils::GetTypeFromBuffer(BYTE *pBuffer, UINT uiBufferSize)
{
	FileType tResult = TYPE_UNKNOWN;

	if (35 <= uiBufferSize)
	{
		if (0 == memcmp(pBuffer, "OggS", 4))
		{
			if (0 == memcmp(pBuffer + 29, "vorbis", 6))
			{
				tResult = TYPE_OGGVORBIS;
			}
			else if (0 == memcmp(pBuffer + 28, "fLaC", 4))
			{
				tResult = TYPE_OGGFLAC;
			}
		}
		else if (0 == memcmp(pBuffer, "fLaC", 4))
		{
			tResult = TYPE_FLAC;
		}
	}

	return tResult;
}

int NUtils::StringCopy(TCHAR *dest, const TCHAR *source, int destSizeChars)
{
	int idx = 0;

	if (destSizeChars > 0 && NULL != dest && NULL != source)
	{
		while ((TEXT('\0') != source[idx]) && (destSizeChars > 1))
		{
			dest[idx] = source[idx];
			idx++;
			destSizeChars--;
		}

		dest[idx] = TEXT('\0');
	}

	return(idx);
}

int NUtils::StringCopyConvert(TCHAR *dest, const char *source, int destSizeChars)
{
	int iResult = 0;

	if (destSizeChars > 0 && NULL != dest && NULL != source)
	{
		// Source is UTF-8 encoded. Need to translate it to UNICODE and then, if we're not a UNICODE build,
		// translate that down to ASCII.
		// Win95 only supports CP_UTF8 with the Microsoft Layer for Unicode installed. Care factor zero.

		int cchwcBuffer = strlen(source) + 1;
		WCHAR *szwcBuffer = new WCHAR[ cchwcBuffer ];

		if (0 < MultiByteToWideChar(CP_UTF8, 0, source, -1, szwcBuffer, cchwcBuffer))
		{
#ifdef UNICODE
			iResult = StringCopy(dest, szwcBuffer, destSizeChars);
			TODO: "..." truncation in the UNICODE build case.
#else
			iResult = WideCharToMultiByte(CP_ACP, 0, szwcBuffer, -1, dest, destSizeChars, NULL, NULL);
#endif
		}

		delete [] szwcBuffer;
	}

	if (0 >= iResult && dest && 0 < destSizeChars)
	{
		if (ERROR_INSUFFICIENT_BUFFER == GetLastError())
		{
			if (destSizeChars >= 4)
			{
				dest[destSizeChars-4] = TEXT('.');
				dest[destSizeChars-3] = TEXT('.');
				dest[destSizeChars-2] = TEXT('.');
			}
			dest[destSizeChars-1] = TEXT('\0');
		}
		else
		{
			dest[0] = TEXT('\0');
		}
	}

	return iResult;
}

char *NUtils::StringExtract(BYTE *source, DWORD dwSourceSizeBytes)
{
	char *szResult = NULL;

	if (NULL != source && dwSourceSizeBytes > 0)
	{
		szResult = new char[ dwSourceSizeBytes+2 ];
		if (NULL != szResult)
		{
			memcpy(szResult, source, dwSourceSizeBytes);
			szResult[dwSourceSizeBytes  ] = '\0';
			szResult[dwSourceSizeBytes+1] = '\0'; // paranoid about MBCS badness.
		}
	}

	return szResult;
}

int NUtils::StringExtractConvert(TCHAR *dest, BYTE *source, int destSizeChars, DWORD dwSourceSizeBytes)
{
	int iResult = 0;

	if (destSizeChars > 0 && NULL != dest && NULL != source && dwSourceSizeBytes > 0)
	{
		char *szExtracted = StringExtract(source, dwSourceSizeBytes);

		if (NULL != szExtracted)
		{
			iResult = StringCopyConvert(dest, szExtracted, destSizeChars);

			delete [] szExtracted;
		}
	}

	return iResult;
}

bool NUtils::CopyVersionNumber(DWORD *pdwHigh, DWORD *pdwLow, TCHAR *szItemName, BYTE *pLang, const LPVOID pVerData)
{
	bool bResult = false;

	TCHAR szVerPath[128];
	_stprintf(szVerPath, _T("\\StringFileInfo\\%02x%02x%02x%02x\\%s"), pLang[1], pLang[0], pLang[3], pLang[2], szItemName);

	LPVOID pBuffer;
	UINT bufLen;
	int v0,v1,v2,v3;

	if (VerQueryValue(pVerData, szVerPath, &pBuffer, &bufLen)
	&&	4 == _stscanf(reinterpret_cast<TCHAR *>(pBuffer), _T("%d, %d, %d, %d"), &v0, &v1, &v2, &v3))
	{
		*pdwHigh = ((v0<<16)|v1);
		*pdwLow  = ((v2<<16)|v3);

		bResult = true;
	}

	return(bResult);
}

bool NUtils::CopyVersionString(TCHAR *szDestBuffer, UINT uiDestBufferSize, TCHAR *szItemName, BYTE *pLang, const LPVOID pVerData)
{
	bool bResult = false;

	if (NULL == szDestBuffer)
	{
		bResult = true;
	}
	else if (2 < uiDestBufferSize)
	{
		TCHAR szVerPath[128];
		_stprintf(szVerPath, _T("\\StringFileInfo\\%02x%02x%02x%02x\\%s"), pLang[1], pLang[0], pLang[3], pLang[2], szItemName);

		LPVOID pBuffer;
		UINT bufLen;

		if (VerQueryValue(pVerData, szVerPath, &pBuffer, &bufLen))
		{
			_tcsncpy(szDestBuffer, reinterpret_cast<TCHAR *>(pBuffer), uiDestBufferSize - 1);
			szDestBuffer[ uiDestBufferSize - 1 ] = _T('\0'); // strncpy/_tcsncpy sucks.

			bResult = true;
		}
	}

	return(bResult);
}
