#pragma once

class CWin32Wrapper
{
protected:
	NUtils::FileType m_Type;
	ov_callbacks m_vf_callbacks;

	LPDVPFILEINFOMUSIC m_lpMusicInfo; // Used by FLAC callbacks.

public:
	CWin32Wrapper(NUtils::FileType fileType);
	virtual ~CWin32Wrapper();

	BOOL Process(LPDVPFILEINFOMUSIC lpMusicInfo);

protected:
	BOOL processFlac(LPDVPFILEINFOMUSIC lpMusicInfo);
	BOOL processOggVorbis(LPDVPFILEINFOMUSIC lpMusicInfo);

	static void parseComment(LPDVPFILEINFOMUSIC lpMusicInfo, const char *szComment, char *szBufferKey, char *szBufferData, size_t nBufferSize);

	// VorbisFile callback functions.
	// See oggvorbis-win32sdk\doc\vorbisfile\callbacks.html for required callback semantics.

	virtual size_t v_vorbisfile_read_func(void *ptr, size_t size, size_t nmemb) = 0;
	virtual int    v_vorbisfile_seek_func(ogg_int64_t offset, int whence) = 0;
	virtual long   v_vorbisfile_tell_func() = 0;

	// VorbisFile static callback wrappers/stubs

	static size_t s_vorbisfile_read_func(void *ptr, size_t size, size_t nmemb, void *datasource)
	{
		return reinterpret_cast<CWin32Wrapper*>(datasource)->v_vorbisfile_read_func(ptr, size, nmemb);
	}

	static int s_vorbisfile_seek_func(void *datasource, ogg_int64_t offset, int whence)
	{
		return reinterpret_cast<CWin32Wrapper*>(datasource)->v_vorbisfile_seek_func(offset, whence);
	}

	static long s_vorbisfile_tell_func(void *datasource)
	{
		return reinterpret_cast<CWin32Wrapper*>(datasource)->v_vorbisfile_tell_func();
	}

	static int s_vorbisfile_close_func(void *datasource)
	{
		return 0;
	}

	// EasyFLAC (FLAC & OggFLAC) callback functions.

	virtual FLAC__StreamDecoderReadStatus v_easyflac_read_func(const EasyFLAC__StreamDecoder *decoder, FLAC__byte buffer[], unsigned *bytes) = 0;
	void m_easyflac_metadata_func(const EasyFLAC__StreamDecoder *decoder, const FLAC__StreamMetadata *metadata);

	// EasyFLAC static callback wrappers/stubs

	static FLAC__StreamDecoderReadStatus s_easyflac_read_func(const EasyFLAC__StreamDecoder *decoder, FLAC__byte buffer[], unsigned *bytes, void *client_data)
	{
		return reinterpret_cast<CWin32Wrapper*>(client_data)->v_easyflac_read_func(decoder, buffer, bytes);
	}

	static FLAC__StreamDecoderWriteStatus s_easyflac_write_func(const EasyFLAC__StreamDecoder *decoder, const FLAC__Frame *frame, const FLAC__int32 * const buffer[], void *client_data)
	{
		return FLAC__STREAM_DECODER_WRITE_STATUS_ABORT; //FLAC__STREAM_DECODER_WRITE_STATUS_CONTINUE;
	}

	static void s_easyflac_error_func(const EasyFLAC__StreamDecoder *decoder, FLAC__StreamDecoderErrorStatus status, void *client_data)
	{
	}

	static void s_easyflac_metadata_func(const EasyFLAC__StreamDecoder *decoder, const FLAC__StreamMetadata *metadata, void *client_data)
	{
		reinterpret_cast<CWin32Wrapper*>(client_data)->m_easyflac_metadata_func(decoder, metadata);
	}
};
